{***************************************************************************
        Fonction rand() de Maple ( MapleTech Vol 1, Num 1, Spring 94 )
                 Linear congruential random number generator

            a = a1 * a0                a  = 427419 669081
                                       a1 =      4 756877
                                       a0 =         89853
                                       m  = 999999 999989

            X(i+1) = a * X(i) mod m = [(X(i) * a1 mod m) * a0] mod m

            b = b1 * b0                b  = 721609 879133
                                       b1 =        759173
                                       b0 =        950521

            a * b = 1 mod m

            Graine permet de gnrer 32 blocs de 3.125e10 nombres
                 pseudo-alatoires conscutifs indpendants.

  Graine [-32..-1] U [1..32],
      si Graine < 0 la gnration s'effectue
                           ? partir de la fin de bloc en "marche arrire"
      si Graine > 0 la gnration s'effectue
                           ? partir du dbut de bloc en "marche avant"

     ( Prcision du partage 1/1200. Fichier d:\TP\TEST\RANDOM\RECH.PAS )

 ***************************************************************************}

 unit alea ;

 interface

 function  random : double ;

 function  random_(cc : longint) : longint ;

 procedure randomize ;

 procedure randseed(rs : integer) ;

 { rs [-32..-1] U [1..32],
            si rs < 0 la gnration s'effectue en "marche arrire"
            si rs > 0 la gnration s'effectue en "marche avant" }

 {**************************************************************************}

 implementation

 uses    biblio ;

 const   graine : array[0..31] of word =
           (    1, 55338, 20927, 15461, 52168, 54531, 11513, 53024,
            24290, 23742, 45189, 65016, 47601, 17139, 28255, 23589,
            57450,  6556, 48661, 26454, 46911, 60670, 60114, 19056,
            35558, 37554, 21679,  7956, 10277, 12840, 28368, 52439) ;

 var     a1, a0, b1, b0, c1, c0 : longint ;
         mm, xx : comp ;
         im : double ;
         cwd : word ;

 function random : double ;

 begin
   asm
      fild     qword ptr[mm]           { ld m }
      fild     qword ptr[xx]           { X(i) }
      fimul    dword ptr[c1]           { X * a1 }
      fprem                            { (X * a1) mod m }
      fimul    dword ptr[c0]           { ((X * a1) mod m) * a0 }
      fprem                            { (((X * a1) mod m) * a0) mod m }
      fld      st                      { ld X }
      fistp    qword ptr[xx]           { sto X(i+1) }
      fmul     qword ptr[im]           { X / m }
      fst      qword ptr[bp - 08]      { retour r,sultat }
      finit
   end ;
 end ;

 function random_(cc : longint) : longint ;

 begin
   asm
      fldcw    cwd                     { arrondi par troncature }
      fild     qword ptr[mm]           { ld m }
      fild     qword ptr[xx]           { X(i) }
      fimul    dword ptr[c1]           { X * a1 }
      fprem                            { (X * a1) mod m }
      fimul    dword ptr[c0]           { ((X * a1) mod m) * a0 }
      fprem                            { (((X * a1) mod m) * a0) mod m }
      fld      st                      { ld X }
      fistp    qword ptr[xx]           { sto X(i+1) }
      fmul     qword ptr[im]           { X / m }
      fimul    dword ptr[bp + 06]      { n }
      fist     dword ptr[bp - 04]      { retour r,sultat }
      finit
   end ;
 end ;

 procedure randomize ;

 var     timer : word absolute $40:$6c ;

 begin
   xx := graine[timer and $1f] ;
 end ;

 procedure randseed(rs : integer) ;

 begin
   if (abs(rs) > 32) or (rs = 0) then
      arret('Entre incorrecte dtecte par la procdure RANDSEED') ;
   if rs > 0 then
      begin
         c1 := a1 ;
         c0 := a0 ;
         xx := graine[rs - 1] ;
      end
   else
      begin
         c1 := b1 ;
         c0 := b0 ;
         xx := graine[abs(rs) mod 32] ;
      end
 end ;

 begin
   xx  :=              1 ;
   a1  :=        4756877 ;
   a0  :=          89853 ;
   b1  :=         759173 ;
   b0  :=         950521 ;
   mm  := 999999999989.0 ;
   im  :=       1.0 / mm ;
   cwd :=          $0f3f ;
   c1  :=             b1 ;
   c0  :=             b0 ;
 end.

 {***************************************************************************
                           Fonction rand() de Maple
 ***************************************************************************}


